/***************************************************************************
 *
 * ATTRNAME.C - Aliases (user friendly labels) for attribute types.
 * 
 ***************************************************************************/

/***************************************************************************
 *
 * @(#) attrname.c 1.2@(#)
 * 
 ***************************************************************************/

#ifndef lint
static char *sccsid="@(#) attrname.c 1.2@(#)";
#endif

#include "macros.h"
#include "query.h"

/**************************************************
 *
 * Local data definitions.
 * 
 */

typedef struct AttrAlias
{
  AttributeType type;
  STRING alias;

  struct AttrAlias *next;
} *AttrAlias;

/**************************************************
 *
 * Local data.
 * 
 */

static AttrAlias aliases = (AttrAlias) NULL; /* Attribute alias list */

/*****************************************************************************
 * AddAttributeAlias()
 *
 * Add an alias for the given attribute (string representation).
 * Return FALSE if invalid attribute name or alias supplied.
 *
 ****************************************************************************/

BOOLEAN AddAttributeAlias(name, alias)
     STRING name;
     STRING alias;
{
  AttributeType attr;
  AttrAlias newname;

  if (isempty(name) || isempty(alias))
    return FALSE;
  
  attr = str2AttrT(name);
  if (attr == NULLAttrT)
    return FALSE;

  newname = (AttrAlias) malloc(sizeof(struct AttrAlias));
  newname->next = aliases;

  newname->type = attr;
  newname->alias = strdup(alias);

  aliases = newname;
  return TRUE;
} /* AddAttributeAlias */

/*****************************************************************************
 * GetAttributeAlias()
 *
 * Retrieve alias for the given attribute name. Return FALSE if
 * nothing found. If nothing found copy supplied attribute name
 * into alias buffer.
 *
 ****************************************************************************/

BOOLEAN GetAttributeAlias(type, aliasbuf)
     AttributeType type;
     char aliasbuf[];
{
  AttrAlias attralias; 

  *aliasbuf = '\0';
  if (type == NULLAttrT)
    return FALSE;

  for (attralias = aliases;
       attralias != (AttrAlias) NULL;
       attralias = attralias->next)
    {
      if (type == attralias->type)
	{
	  (void) strcpy(aliasbuf, attralias->alias);
	  return TRUE;
	}
    }

  (void) strcpy(aliasbuf, attr2name(type, OIDPART));
  return FALSE;
} /* GetAttributeAlias */
