/***************************************************************************
 *
 * @(#) configure.y 1.5@(#)
 *
 ***************************************************************************/


%{
#ifndef lint
static char *sccsid="@(#) configure.y 1.5@(#)";
#endif

#include "types.h"
#include "init.h"
#include "attrname.h"
#include "attrread.h"
#include "query.h"

int yylex();
void exit();

extern FILE *confile;
extern int TailorLineNumber;
static char lexeme[255];

%}
%start configure

%union
{
  char strval[255];
  int symbol;
  stringCell string_list;
  entryList entry_list;
  int integer;
}

%token <symbol> UFN_PATH ENTRYDB ATTR_NAME READ_TYPE
%token <strval> STRINGVAL

%type <integer> 	ufn_bound
%type <entry_list>      entry_list

%%

/***************************************************
 *
 * Main rule.
 *
 */

 configure:	var_list
      	   	;

/***************************************************
 *
 * The configure options.
 *
 */

 var_list:	STRINGVAL var_list			{;}

		| ufn_path_option var_list		{;}

		| entry_db_option var_list		{;}
	
		| attr_name_option var_list		{;}
	
		| read_option var_list			{;}
	
		|					{;}
		;

/***************************************************
 *
 * Entry database.
 * 
 */

 entry_db_option:	ENTRYDB ':' STRINGVAL ':' STRINGVAL
			  {
			    AddDBEntry($3, $5);
			  }

/***************************************************
 *
 * Directory configuration rules.
 *
 */

/***************************************************
 *
 * UFN paths
 *
 */

 ufn_path_option:	UFN_PATH ':' ufn_bound ',' ufn_bound ':' entry_list
			  {
			    QyUFNAddPath($3, $5, $7);
			  }

			| UFN_PATH ':' ufn_bound ':' entry_list
			  {
			    QyUFNAddPath($3, _QyUFNUnbounded, $5);
			  }
			;

 ufn_bound:	STRINGVAL 
		  {
		    if (!isdigit(*($1)))
		      $$ = _QyUFNUnbounded;
		    else
		      {
			int c;

			long strtol();

			c = (int) strtol((char *) $1, (char **) NULL, 10);
			$$ = c;
		      }
		  }

		| '+'
 	  	  {
		    $$ = _QyUFNPlus;
		  }

		;

 entry_list:	STRINGVAL
		  {
		    STRING dn;
		    $$ = NULLEntryList;
	
		    dn = GetDBEntry($1);
		    (void) dn_list_insert(dn, &($$), NULLAttrT);
		  }

		| STRINGVAL ',' entry_list 
		  {
		    STRING dn;
		    $$ = $3;
	
		    dn = GetDBEntry($1);
		    (void) dn_list_insert(dn, &($$), NULLAttrT);
		  }

		;

/**************************************************
 *
 * Read user friendly attribute names.
 * 
 */

 attr_name_option:	ATTR_NAME ':' attr_name_list {;}

 attr_name_list:        STRINGVAL STRINGVAL attr_name_list
                          {
                            (void) AddAttributeAlias($1, $2);
                          }

                        | ',' STRINGVAL STRINGVAL attr_name_list
                          {
                            (void) AddAttributeAlias($2, $3);
                          }

                        |		
                          {;}
                        ;

/**************************************************
 *
 * Attributes to read.
 * 
 */

 read_option:		READ_TYPE ':' read_attr_list {;}
 
 read_attr_list:        STRINGVAL read_attr_chain
                          {
                            AddDisplayAttribute($1);
                          }
                        | '-' STRINGVAL read_attr_chain
                          {
                            AddDisplayAttribute($2);
                          }
                        |
                        ;
 
 read_attr_chain:       ',' STRINGVAL read_attr_chain
                          {
                            AddDisplayAttribute($2);
                          }
                        | ',' '-' STRINGVAL read_attr_chain
                          {
                            AddDisplayAttribute($3);
                          }
                        |
                        ;


%%

/***************************************************
 *
 * The lexer.
 *
 */

int yylex()
{
  register int c;
  int count = 0;
  FILE *lexfile = confile;
  register STRING lexptr;
  BOOLEAN space;

  extern int _filbuf(), fseek();
  
  lexeme[0] = '\0';
  TailorLineNumber++;
  
  while (isspace((char) (c = getc(lexfile))))
    {
      if (c == EOF)
	return 0;
    }
  
  lexeme[count++] = c;
  
  switch (c)
    {
    case '#':

      while ((c = getc(lexfile)) != '\n' && c != EOF)
	;

      return yylex();

    case ':':
    case '(':
    case ')':
    case ',':
    case '{':
    case '}':
    case '-':
    case '+':

      lexeme[count] = '\0';
      return c;

    case '\"':

      count = 0;
      space = FALSE;

      while ((c = getc(lexfile)) != EOF && c != '\"')
	{
	  if (space == FALSE)
	    {
	      lexeme[count++] = (char) c;
	      
	      if (isspace((char) c))
		space = TRUE;
	    }
	  else if (!isspace((char) c))
	    {
	      lexeme[count++] = (char) c;
	      space = FALSE;
	    }
	}
    
      if (c == '\"')
	{
	  lexeme[count] = '\0'; 

	  (void) strcpy(yylval.strval, lexeme);
	  return STRINGVAL;
	} else
	  return 0;
    }

  while ((c = getc(lexfile)) != EOF && 
	 ! (c == '('
	    || c == ')'
	    || c == ':'
	    || c == '{'
	    || c == '}'
	    || c == ','
	    || c == '\"'
	    || isspace((char) c)))
    lexeme[count++] = (char) c;
  
  if (c == EOF)
    return 0;
  
  (void) fseek(lexfile, (long) -1, 1);
  
  lexeme[count] = '\0';
  lexptr = (STRING) lexeme;

  switch (*lexptr)
    {
    case 'a':
    case 'A':
      
      if (lexequ("AttrName", lexptr) == 0)
	{
	  yylval.symbol = ATTR_NAME;
	  return ATTR_NAME;
	}

    case 'e':
    case 'E':
      
      if (lexequ("EntryDB", lexptr) == 0)
	{
	  yylval.symbol = ENTRYDB;
	  return ENTRYDB;
	}

    case 'd':
    case 'D':
      
      if (lexequ("DisplayAttribute", lexptr) == 0)
	{
	  yylval.symbol = READ_TYPE;
	  return READ_TYPE;
	}

    case 'u':
    case 'U':
      
      if (lexequ("ufnPath", lexptr) == 0)
	{
	  yylval.symbol = UFN_PATH;
	  return UFN_PATH;
	}

      break;
    }

  (void) strcpy(yylval.strval, lexeme);
  return STRINGVAL;
}

/* ARGSUSED */
int yyerror(err)
     char *err;
{
  (void) fprintf(stderr, "Parse error.\n");
  exit(1);
}
