/***************************************************************************
 *
 * @(#) dntree.c 1.3@(#)
 *
 ***************************************************************************/
 
#ifndef lint
static char *sccsid="@(#) dntree.c 1.3@(#)";
#endif


/*
** DNTree.c - Paul Wain
**
** This file includes the routines for setting up a tree of the information
** returned by a "Browse" command. This tree is used to sort the information
** into a list of classes and thus display the list as a set of end names
** BUT grouped by common denominators, rather than just a straight list as
** returned by the QYBrowse function.
**
** To set up the list an EntryList (see entrylist.h) is passed to a setup
** routine which in turn translates this into a tree structure.
**
** A display routine is also provided to convert the tree into a readable
** format on the scrollbar window.
**
** Finally a rountine is also provided to convert the entry in the tree
** into the correct entrylist entry, although technically speeking entry1
** in the entry list should be the 1st entry arrived at in the tree and the
** same goes for the nth entry.
**
*/

#include "sequence.h"
#include "dntree.h"
#include "textdpy.h"
#include "types.h"
#include "query.h"
#include "show.h"
#include "entrylist.h"
#include "history.h"

/*
** A few misc bits which should really be in a header file
*/

void Free_dn_tree();
void Add_dn_tree();
int  Add_dn_tree_entry();
void Add_dn_tree_cell();
void Display_dn_tree();
void Do_display_dn_tree();
void getword();
char *Copy_string();
void Display_selectable();
void Display_not_selectable();
void Show_dn_tree_entry();
int Search_and_show_dn_tree_entry_number();

DNTree dn_tree;
DNTree treeptr;
int    dn_tree_exists=0;
int    entry_number=0;

#define DNTree_Alloc() (DNTree)malloc(sizeof(struct TREEDN))

#define NOTHING 0
#define DOWN 1
#define LEVEL 2

#define SELECTABLE 1
#define NOT_SELECTABLE -1

extern char *_setbold, *_clearbold;

/*
** Initialise the base of the DNTree 1st time around.
** This always affects "dn_tree" (29/09/93 1.1)
*/

void Init_dn_tree()
{
    dn_tree = (DNTree)DNTree_Alloc();
    dn_tree->attribute=Copy_string("root");
    dn_tree->value=Copy_string("root");
    dn_tree->level = NULL;
    dn_tree->down = NULL; 
    dn_tree->selectable = NOT_SELECTABLE;
    dn_tree_exists=1;
}

/*
** Routine to initialise the DN Tree (28/9/93 - 1.0)
*/

void Create_dn_tree(entrypointer)
entryList entrypointer;
{
char *buffer;

    AddData("Sorting results - please wait!");
    Scrollbar('\0');

    if (dn_tree_exists!=0) {
	    Free_dn_tree(dn_tree);
	}

    Init_dn_tree();

    for (;
	 entrypointer->next != NULL;
	 entrypointer = entrypointer->next) {

		treeptr=dn_tree;

		if (treeptr->down != NULL) {
			treeptr=treeptr->down;
			}

		buffer=Copy_string(entrypointer->string_dn);

	 	Add_dn_tree(buffer,entrypointer->string_dn);
		free((char *) buffer);
		}

/*
** This is a hack to ensure the last entry gets added :)
*/
	if (entrypointer->string_dn != NULL) {
		treeptr=dn_tree;

        	if (treeptr->down != NULL) {
                	treeptr=treeptr->down;
                	}
 
        	buffer=Copy_string(entrypointer->string_dn);
 
        	Add_dn_tree(buffer,entrypointer->string_dn);
        	free((char *) buffer);
		}
}

/*
** Add_dn_tree takes the base tree pointer and attempts to add a new
** entry at the correct point within the tree - (29/9/93 - 1.1)
*/

void Add_dn_tree(string,string_dn)
char   *string;
char   *string_dn;
{
char new_string[BUFLEN];
int  result=0;
DNTree tp;

/*
** What we need to do here is split up the string into the various parts
** adding them to the tree on the fly so to speak.
*/

    tp = dn_tree;

    getword(new_string,string,'@');

    (void) Add_dn_tree_entry
        (&tp, new_string, string, string_dn);

}

/*
** This routine works to the bottom of the dn_tree and works its way back
** up the tree, freeing each cell as it does so...
**
** Recursion is the name of the game here again!
*/

void Free_dn_tree(treeptr)
DNTree treeptr;
{
if (treeptr->down != NULL) {
	Free_dn_tree(treeptr->down);
	treeptr->down = NULL ;
	}

if (treeptr->level != NULL) {
	 Free_dn_tree(treeptr->level);
	 treeptr->level = NULL ; 
	 }

free((char *)treeptr->attribute);
free((char *)treeptr->value);
free((char *)treeptr->string_dn);
free((char *)treeptr);
dn_tree_exists=0;
}


int Add_dn_tree_entry(tp,passed_string,dn_left,string_dn)
DNTree *tp;
char *passed_string, *dn_left;
char *string_dn;
{
char attribute[BUFLEN], string[BUFLEN], *buffer, buffer2[BUFLEN];
int result=DOWN;
RDN rdn,treerdn;
DNTree level;

    buffer=Copy_string(passed_string);
    getword(attribute,buffer,'=');
    getword(string,buffer,'\0');
    free(buffer);

    buffer=Copy_string(dn_left);
    getword(buffer2,buffer,'@');

if (dn_left[0]=='\0')
  /**************************************************
   *
   * This is the final RDN of the DN, so compare
   * against things at the same level.
   * 
   */
  {
    rdn = str2rdn(passed_string);
    
    for (level = (*tp)->level; level != (DNTree) NULL; level = level->level)
      {
	buffer2[0]='\0';
	strcat(buffer2,(*tp)->attribute);
	strcat(buffer2,"=");
	strcat(buffer2,(*tp)->value); 
	treerdn = str2rdn(buffer2);
	if (rdn_cmp(rdn, treerdn) == 0) 
	  {
	    result = NOTHING;
	  }

	rdn_free(treerdn);
      }
    
    rdn_free(rdn);

    if ((level == (DNTree) NULL) && (result != NOTHING))
      {
    	Add_dn_tree_cell(&((*tp)->down),
    		attribute, string, SELECTABLE,string_dn);
        result = DOWN;
      }
    else if (result != NOTHING) {
       	Add_dn_tree_cell(&((*tp)->level),
		 attribute, string, SELECTABLE,string_dn);
       	result = LEVEL;
       	}

    free(buffer);
    return result;
  }
else
  /**************************************************
   *
   * Not the final RDN, compare against children
   * (i.e recursive call).
   * 
   */
  {
    free(buffer);
    tp = &((*tp)->down);

    if (*tp == (DNTree) NULL)
        {
    	getword(buffer2,dn_left,'@');
	Add_dn_tree_cell(tp, attribute, string, NOT_SELECTABLE,string_dn);
	Add_dn_tree_entry(tp, buffer2, dn_left, string_dn); 
	return NOTHING;
        }
    else
        {
    	rdn = str2rdn(passed_string);

    	/* for (level = (*tp)->level;
			level != (DNTree) NULL; level = level->level) */

    	for ( ; *tp != (DNTree) NULL; tp = &((*tp)->level))
      		{
		buffer2[0]='\0';

		strcat(buffer2,(*tp)->attribute);
 		strcat(buffer2,"=");
		strcat(buffer2,(*tp)->value);

		treerdn = str2rdn(buffer2);

		if (rdn_cmp(rdn, treerdn) == 0) 
	  		{
	    		result = NOTHING;
			break;
	  		}

		rdn_free(treerdn);
      		}
    
    	if (*tp == (DNTree) NULL) {
		Add_dn_tree_cell(tp,attribute,string,NOT_SELECTABLE,
				string_dn);
		} 

    	rdn_free(rdn);

    	getword(buffer2,dn_left,'@');
	Add_dn_tree_entry(tp, buffer2, dn_left, string_dn); 
	return NOTHING;
        }
    }
}

void Add_dn_tree_cell(tp,attribute,string,selectable,string_dn)
DNTree *tp;
char string[BUFLEN];
char attribute[BUFLEN];
int  selectable;
char *string_dn;
{
DNTree new_cell;

    new_cell = DNTree_Alloc();
    new_cell->attribute=Copy_string(attribute);
    new_cell->value=Copy_string(string);
    new_cell->level = *tp;
    new_cell->down = NULL; 
    new_cell->selectable = selectable;
    
    if (selectable==SELECTABLE) {
	new_cell->string_dn=Copy_string(string_dn);
	}
    else {
	new_cell->string_dn=NULL;
	}

    *tp = new_cell;
}

/*
** Get word - used for splitting up strings :)
*/

void getword(word, line, stop)
char *word;
char *line;
char stop;
{
    int x = 0,y;
 
    for(x=0;((line[x]) && (line[x] != stop));x++)
        word[x] = line[x];
 
    word[x] = '\0';
    if(line[x]) ++x;
    y=0;
 
    while(line[y++] = line[x++]);
}

/*
** This final routine is used to actually display the tree created to
** the user in a more readable form than just a load of UFN's. 
**
** Hopefully :)
**
*/

void Display_dn_tree()
{
    treeptr=dn_tree->down; /* to skip root reference level */
    entry_number=1;
    Do_display_dn_tree(treeptr,"");
    AddData("\n");
}

void Do_display_dn_tree(treeptr,padding)
DNTree treeptr;
char   *padding;
{
char new_padding[BUFLEN];
int  displayed=0;

    strcpy(new_padding,padding); 
    strcat(new_padding,"     ");

	if ( treeptr->down != NULL ) {
		if ( treeptr->selectable == SELECTABLE ) {
			Display_selectable(entry_number,
					   padding,
					   treeptr->value);
			entry_number++;
			displayed=1;
			}
		else {
			Display_not_selectable(padding,
					       treeptr->value);
			displayed=1;
			}
		Do_display_dn_tree(treeptr->down,new_padding);
		}

	if ( treeptr->level != NULL ) {
		Do_display_dn_tree(treeptr->level,padding);
		}

	if ( displayed == 0 ) {
		if ( treeptr->selectable == SELECTABLE ) {
			Display_selectable(entry_number,
                                           padding,
                                           treeptr->value);
			entry_number++;
			}
		else {
			Display_not_selectable(padding,
					       treeptr->value);
			}
		}
}

void Display_selectable(number,padding,string)
int number;
char *padding, *string;
{
char buffer[BUFLEN];

	buffer[0]='/0';

        sprintf(buffer,"%s%2d - %s\n",padding,number,string);

	AddData(buffer);
}

void Display_not_selectable(padding,string)
char *padding, *string;
{
char buffer[BUFLEN];

        sprintf(buffer,"%s%s\n",padding,string);
	AddData(buffer);
}

char *Copy_string(string)
char *string;
{
char *new_string;

    if (string == NULLCP) return NULLCP;
	   
    new_string = (char *)malloc(strlen(string) + 1);
    (void) strcpy(new_string, string);
		
    return new_string;
}

/*
** void Show_dn_tree_entry();
**
** Routine to get the CORRECT entry from the dn_tree after it has been 
** ordered by department and name etc... Very close in origin to the
** display_dn_tree routine :)
**
*/

void Show_dn_tree_entry(index)
int index;
{
Search_and_show_dn_tree_entry_number(dn_tree,index,0);
}

int Search_and_show_dn_tree_entry_number(treeptr,target,current)
DNTree treeptr;
int target;
int current;
{

	if (current < target) /* Makes sure we dont do unnecessary work! */
		{
		if ( treeptr->down != NULL ) {
			current=Search_and_show_dn_tree_entry_number(
				treeptr->down,target,current);
			}
		if ( treeptr->level != NULL ) {
			current=Search_and_show_dn_tree_entry_number(
				treeptr->level,target,current);
			}
		}

	if ( treeptr->selectable == SELECTABLE ) {
		current++;
		}

	if (current == target) {
		Show(treeptr->string_dn);
		AddHistory("Select entry:",treeptr->string_dn);
		current++ ; /* hack to prevent multiple unwanted reads */
		}

	return current;
}
