/***************************************************************************
 *
 * @(#) dua.c 1.2@(#)
 *
 ***************************************************************************/
 
#ifndef lint
static char *sccsid="@(#) dua.c 1.2@(#)";
#endif


#include "query.h"
#include "logger.h"
#include "types.h"
#include "dua.h"

/**************************************************
 *
 * Local functions.
 * 
 */

static BOOLEAN DirectoryWork();

/**************************************************
 *
 * Global data.
 * 
 */

char username[LINESIZE], userpassword[LINESIZE];

/**************************************************
 *
 * External data.
 * 
 */

extern char *dsa_address;

/**************************************************
 *
 * External procedures.
 * 
 */

#ifndef NO_STATS
extern LLog *log_stat;
#endif 

/*****************************************************************************
 * DirectoryWait()
 *
 * A request has been invoked. Call the supplied callbacks when reports or
 * results come in.
 *
 ****************************************************************************/

void DirectoryWait(requestId, rescb, repcb)
     int requestId;
     RequestCallback rescb;
     RequestCallback repcb;
{
  fd_set assoc, writefds, exceptfds;
  BOOLEAN done;

  extern int dsap_ad;
  
  done = FALSE;
  while (done == FALSE)
    {
      /* Wait for something to come in */
      do
	{
	  FD_ZERO(&assoc);
	  FD_ZERO(&writefds);
	  FD_ZERO(&exceptfds);
	  FD_SET(dsap_ad, &assoc);
	}
      while (select(getdtablesize(), &assoc, &writefds, &exceptfds, NULL) <=0);

      /* Process incoming result */
      done = !DirectoryWork(requestId, rescb, repcb);
    }

  QyRequestComplete(requestId);
} /* RequestInvoked */

/*****************************************************************************
 * DirectoryWork()
 *
 * Process an incoming directory result PDU. Call the appropriate
 * user callback procedure. If request has completed return FALSE.
 *
 ****************************************************************************/

static BOOLEAN DirectoryWork(requestId, rescb, repcb)
     int requestId;
     RequestCallback rescb;
     RequestCallback repcb;
{
  int *reslist, *replist;
  int rescount, repcount, count;

  requestId=requestId; /* stop LINT complaining :) */

  QyPoll(&reslist, &rescount, &replist, &repcount);
  
  if (rescount > 0)
    {
      /**************************************************
       *
       * Process completed requests. Loop not strictly
       * necessary as, at present, should only ever
       * receive one at a time.
       * 
       */
      
      for (count = 0; count < rescount; count++)
	{
	  (*rescb)(reslist[count]);
	}

      return FALSE;
    }

  if (repcount > 0)
    {
      /**************************************************
       *
       * Process request reports. Loop not strictly
       * necessary as, at present, should only ever
       * receive one at a time.
       * 
       */
      
      for (count = 0; count < repcount; count++)
	{
	  (*repcb)(replist[count]);
	}
    }

  /* All done! */
  if (reslist != (int *) NULL)
    free((char *) reslist);

  if (replist != (int *) NULL)
    free((char *) replist);

  return TRUE;
} /* DirectoryWork */

