/***************************************************************************
 *
 * ENTRYLIST.C - Store and access lists of entries returned by
 *		 search or browse operations.
 * 
 ***************************************************************************/

/***************************************************************************
 *
 * @(#) entrylist.c 1.4@(#)
 *
 ***************************************************************************/
 
#ifndef lint
static char *sccsid="@(#) entrylist.c 1.4@(#)";
#endif


#include "types.h"
#include "query.h"
#include "show.h"
#include "textdpy.h"
#include "entrylist.h"
#include "history.h"
#include "dntree.h"

/**************************************************
 *
 * Global and Local data.
 * 
 */

entryList CurrentList = NULLEntryList;
extern DNTree dn_tree;

int selectentry_mode = NORMAL ;

/*****************************************************************************
 * SetEntryList()
 *
 * Set the current list of entries. 
 *
 ****************************************************************************/

void SetEntryList(list)
     entryList list;
{
  dn_list_free(&CurrentList);
  CurrentList = NULLEntryList;
  dn_list_copy(list, &CurrentList);
} /* SetEntryList */

/*****************************************************************************
 * SelectEntry()
 *
 * Select (read & show) the indexed entry from the list. Return FALSE
 * if given index is out of bounds.
 *
 ****************************************************************************/

BOOLEAN SelectEntry(idx)
     int idx;
{
   int count;
   entryList entryptr = CurrentList;
   char buff[BUFLEN];
  
  /* Index starts at 1 */
   if (--idx < 0)
     return FALSE;
  
   for (count = 0; count < idx; count++)
     {
       if (entryptr == NULLEntryList)
   	break;
  
      entryptr = entryptr->next;
    }
  
   if (entryptr == NULLEntryList)
     return FALSE;
   
   QyPrintFriendlyDN(entryptr->string_dn,buff);
   AddHistory("Select entry:",entryptr->string_dn);
   
   Show(entryptr->string_dn);
   return TRUE; 
} /* SelectEntry */

/*****************************************************************************
 * ShowEntryList()
 *
 * Display the current entry list in the data window. Return FALSE if
 * current entry list is empty.
 *
 ****************************************************************************/

BOOLEAN ShowEntryList()
{
  if (CurrentList == NULLEntryList)
    return FALSE;

  Create_dn_tree(CurrentList);

  ClearData();

  Display_dn_tree();

  selectentry_mode = DN_TREE_MODE ;

  AddData("\n");
  Scrollbar('\0');

  return TRUE;
} /* ShowEntryList */

/*****************************************************************************
 * GotoEntry()
 *
 * 
 *
 ****************************************************************************/

void GotoEntry(stridx)
     STRING stridx;
{
  STRING numstr;
  int idx;

  extern HISTORY CommandHistory[MAX_HISTORY];
  extern void Clear_goto_number();
  extern void Show_dn_tree_entry();

  numstr = TidyString(stridx);
  idx = atoi(numstr);

  switch (selectentry_mode) {
	case NORMAL:
  		(void) SelectEntry(idx); /* #### Check return value? #### */
		break;

	case HISTORY_MODE:
		/* idx; -- Womder what this was - amy ideas? */
		if (idx<MAX_HISTORY) 
		    if ( CommandHistory[idx-1].DN[0] != '\0' ) {

  			AddHistory("Review entry:",
				   CommandHistory[idx-1].DN);

			(void) Show(CommandHistory[idx].DN);
			}
		selectentry_mode = NORMAL;
		break;

	case DN_TREE_MODE:
		Show_dn_tree_entry(idx);
		break;
		
	case CROSS_REFERENCE:
		selectentry_mode = NORMAL;
		break;

	}

Clear_goto_number();

} /* GotoEntry */

