/* help.c - Display of assorted help texts */

/***************************************************************************
 *
 * @(#) help.c 1.2@(#)
 *
 ***************************************************************************/
 
#ifndef lint
static char *sccsid="@(#) help.c 1.2@(#)";
#endif


#include <stdio.h>
#include <string.h>
#include "help.h"
#include "sequence.h"
#include "dua.h"

extern stringList DataList;

/******************************************************************************
 * InitHelp()
 *
 * Display a few basic lines of help upon startup
 ******************************************************************************/

void InitHelp()
{
(void) GetHelpFile("quickhelp", REFRESH);
}



/******************************************************************************
 * HelpMain()
 *
 * Display the main help page and prompt the user to select an option from
 * it. Use the helpdir as a place to look up the help pages.
 ******************************************************************************/

void HelpMain()
{
char ch;		/* input charecter. */
int  loop=1;		/* general purpose integer */
stringList oldscr;	/* The old data window! */
char oldmessage[BUFLEN];/* The old message header */

/*
 * First of all grab the old screen and message text so we can do a clean
 * return...
 *
 * See note on CopyDataList() at the function itself!
 */

(void) CopyDataList(&oldscr,DataList);

(void) GetMessage(oldmessage);

/*
 * Then we just want to open the main help page: HELPDIR/help
 */

(void) GetHelpFile("help", NO_REFRESH);


/*
 * And tell the user that they are in help mode...
 */

(void) SetMessage("Help mode. Press <SPACE> to return to Xslu or <KEY> for help");

/*
 * Now add the line of text underneath it!
 */

AddData("     Press the key for help on that key, or <SPACE> To return to XSLU\n");
AddData("\n");
Scrollbar('\0');

while (loop) {

	ch = (char) getchar();

	if (isupper(ch))
		ch = tolower(ch);

/*
 * Allow the user to refresh the screen with a ^L input
 */

	switch (ch)
		{
		case '\014':
			Redraw();
			Scrollbar('\0');
			break;
	
		case ' ':
			loop=0;
			ClearData();
			continue;
			break;

		default:
			GetHelp(ch);
		}
	}

/*
 * Restore the old settings...
 */

(void) CopyDataList(&DataList,oldscr);

(void) SetMessage(oldmessage);

Scrollbar('\0');
}

/******************************************************************************
 * DoHelp()
 *
 * Take the char passed to the function and return the help page for that char
 * from the correct file.
 ******************************************************************************/

void GetHelp(ch)
	char ch;
{
int lastline=0;	/* Do we need to add a last line to the screen? */
switch (ch)
	{
      	case '1':
       	case '2':
       	case '3':
       	case '4':
       	case '5':
       	case '6':
       	case '7':
       	case '8':
       	case '9':
	case '0':
	case '#':
		GetHelpFile("number",NO_REFRESH);
		lastline=1;
       		break;

       	case '[':
       	case ']':
       		Scrollbar(ch);
		break;

	case 'b':
		GetHelpFile("browse",NO_REFRESH);
		lastline=1;
		break;

	case 'o':
		GetHelpFile("owner",NO_REFRESH);
		lastline=1;
		break;

	case 'h':
		GetHelpFile("history",NO_REFRESH);
		lastline=1;
		break;

	case '?':
		GetHelpFile("help",NO_REFRESH);
		AddData("     Press the key for help on that key, or <SPACE> To return to XSLU\n");
		AddData("\n");
		Scrollbar('\0');
		break;
	
	case 'q':
		GetHelpFile("quit",NO_REFRESH);
		lastline=1;
		break;

	case 'n':
		GetHelpFile("searching",NO_REFRESH);
		lastline=1;

	}

if (lastline==1) {
	AddData(" Press the key for help on that key, <?> for help menu, or\n");
	AddData(" <SPACE> to return to XSLU\n");
	AddData("\n");
	Scrollbar('\0');
	}
}


/******************************************************************************
 * GetHelpFile()
 *
 * Opens a help file and writes its contents to the main output window, just 
 * using one screen refresh. Aditional parameter specifies not to do a screen
 * update for if the help routine above wishes to suffix the file with a line
 * or two of TEXT
 ******************************************************************************/

void GetHelpFile(helpfile, refresh)
	int  refresh;
	char *helpfile; /* the help file we want to open */
{
char buff[BUFLEN];      /* Buffer for text to be loaded into */
char filename[BUFLEN];  /* The full file name of the help file */
FILE *input_file;       /* file pointer to file we are reading */

/*
 * First of all we want to open the help page: HELPDIR/filename
 * Giving an error if we cant find it...
 */

if (testing == TRUE) { 
	(void) strcpy(filename,HELPDIR);
}
else {	
	(void) strcpy(filename,ETCDIR);
	(void) strcat(filename,"slu/help/");
	}

(void) strcat(filename,helpfile);

if ((input_file=fopen(filename,"r"))==NULL) {
	ClearData();
        sprintf(buff,"Error: Unable to display help for: %s\n",filename);
        AddData(buff);
	AddData("\n");
        Scrollbar("\0");
        return ;
        }

/*
 * Clear the output window ready for "cat"ing help file to it...
 */

ClearData();

/*
 * Read in the input buffer to the 2nd screen ready to disply it
 */

while (fgets(buff,BUFLEN,input_file) != (char *)NULL){

        AddData(buff);
        }

/*
 * Display the 2nd screen
 */

AddData("\n"); 	 /* Bodge to ensure the last line is displayed! */

/*
 * Close the file handler to preserve descriptors
 */

(void) close(input_file);

/*
 * If required, redisplay the screen
 */

if (refresh) Scrollbar('\0');

}

/*******************************************************************************
 * CopyDataList()
 *
 * This copies the contents of the Data window into a new linked list. In most
 * (all?) cases this is done to temporarily store the data window for say,
 * help text to be displayed. 
 *
 * The routine actually coppies the list BACKWARDS but this is okay since 
 * as mentioned above the list is generally coppied to a temp list and then
 * coppied back to DataList, so we are effectivly doing: BACKWARDS(BACKWARDS)
 * which is, of cause, FORWARDS!
 *
 * SeqAddString is done to avoid MALLOC problems :)
 *******************************************************************************/

void CopyDataList(new,old)
stringList *new,old;
{
stringList head = old;

*new = NULLStrCell;

while( head != NULLStrCell ) {
	SeqAddString(head->string,new);
	head=head->next;
	}
}
