/***************************************************************************
 *
 * HISTORY.C - Search history functions... Add, List and Initialise.
 * 
 ***************************************************************************/

/***************************************************************************
 *
 * @(#) history.c 1.2@(#)
 *
 ***************************************************************************/
 
#ifndef lint
static char *sccsid="@(#) history.c 1.2@(#)";
#endif


#include <time.h>
#include "widget.h"
#include "types.h"
#include "history.h"
#include "entrylist.h"

#define NULLHistory {"", "" ,"", 0}

HISTORY CommandHistory[MAX_HISTORY];
extern char *local_dit;

/*****************************************************************************
 * ShowHistory()
 *
 * Show a list of the searches entered so far and set the list select mode
 * up so that a given search can be repeated.
 *
 ****************************************************************************/

void ShowHistory()
{
int loop=0;			/* loop control variable thang 	*/
char buff[BUFLEN];		/* buffer to print into		*/
char buff2[BUFLEN];		/* buffer to print into		*/
char tempbuf[BUFLEN];		/* temporary buffer		*/

extern int selectentry_mode;	/* Select mode for Goto #	*/

/*
 * Clear the data window and set the History message.
 */

ClearData();
SetMessage("Command history.");

/*
 * Look to see if a search has been carried out yet, and if it hasn't then
 * explain that to the user
 */

if(CommandHistory[0].DN[0] == '\0') {
	AddData("\n\n Error: No search commands entered yet.\n\n");
	}
else { 

/*
 * Otherwise list the commands. First of all add a heafer to the window, then
 * loop around displaying the history entries.
 */

	sprintf(buff,"\n   %-7s %-30s\n\n","Time","Command");
	AddData(buff);
	while( (CommandHistory[loop].DN[0] != '\0')
					&& (loop < MAX_HISTORY-1) )
		{

/*
 * Convert the DN to the nearest (possibly non-unique) UFN.. and set up
 * a string displaying the search type and the UFn... only 64 chars are used
 * hence the strncpy. The '\0' is needed to allow for strncpy running out of
 * chars when it copies... blame string.h :)
 */
		QyPrintFriendlyDN(CommandHistory[loop].DN, buff2);

		sprintf(buff,"%s %s",
			CommandHistory[loop].Command,
			buff2);

		strncpy(tempbuf, buff, 64);
		tempbuf[64]='\0';

/*
 * Then set up the full history line of command number, time it was entered
 * and the search command itself and add it to the data window.
 */
		sprintf(buff,"%2d %-7s %s\n",
			loop+1,
			CommandHistory[loop].Time,
			tempbuf);

		AddData(buff);
		loop++;
		}
	} 

/*
 * Show the data, and set the Goto Entry mode to History.
 */

Scrollbar('\0');

selectentry_mode = HISTORY_MODE;

} /* ShowHistory */

/*****************************************************************************
 * AddHistory()
 *
 * Add an entry to the history data structure, shifting all the other entries
 * down one.
 *
 ****************************************************************************/

void AddHistory(command, dn)
	char command[BUFLEN];
	char dn[BUFLEN];
{
HISTORY moved[MAX_HISTORY]; /* structure to shuffle the history entries in. */
int	loop;	            /* loop control variable			    */

struct tm *tm;		    /* Time structure 				    */
time_t	t;		    /* Time pointer				    */

extern WIDGET *currwidgets; /* global widget set		            */

/*
 * Work out the current local time
 */

t = time(NULL);
tm = (struct tm *)localtime(&t);

/*
 * Copy the CommandHistory into the moved history structure shifting it up
 * one place so that the new entry can go in position 0
 */

for (loop=1;loop<MAX_HISTORY;loop++) moved[loop]=CommandHistory[loop-1];

/*
 * Now add the new data: search type, DN and the timestamp
 */

(void) strcpy(moved[0].Command,command);
(void) strcpy(moved[0].DN,dn);
(void) strftime(moved[0].Time,BUFLEN-1,"%H:%M",tm);

/*
 * Copy the new structure (moved) back into CommandHistory
 */

for (loop=0;loop<MAX_HISTORY;loop++) CommandHistory[loop]=moved[loop];

}

/*****************************************************************************
 * InitHistory()
 *
 * Initialise the CommandHistrioy structure. Basically just loop around
 * setting the strings to "" and the Valid flag FALSE.
 *
 ****************************************************************************/

void InitHistory()
{
int loop=0;

while (loop<MAX_HISTORY-1) {
	CommandHistory[loop].Command[0]='\0';
	CommandHistory[loop].Time[0]='\0';
	CommandHistory[loop].DN[0]='\0';
	loop++;
	}

if ((local_dit != NULL) && (*local_dit!=NULL))
	AddHistory("Starting Point:",local_dit);
	
}
