/***************************************************************************
 *
 * @(#) main.c 1.5@(#)
 *
 ***************************************************************************/
 
#ifndef lint
static char *sccsid="@(#) main.c 1.5@(#)";
#endif

#include <stdio.h>
#include <signal.h>
#include <varargs.h>

#include "quipu/util.h"
#include "query.h"
#include "dua.h"
#include "help.h"
#include "main.h"
#include "init.h"
#include "version.h"

extern char username[], userpassword[];
extern char *oidtable, *tailfile, *myname;
extern char print_parse_errors;

static void ReadArguments();
int testing = FALSE;
char config_file_name[BUFLEN];

int main(argc, argv)
     unsigned int     argc;
     char    *argv[];
{

  extern void Interact();
  extern void InitWidgets();
  extern void InitHistory();
  extern void InitTermPrint();
  extern void InitDNTree();
  
  print_parse_errors = FALSE;
  quipu_syntaxes();
  
  username[0] = '\0';
  userpassword[0] = '\0';
  config_file_name[0]='\0';
  
  ReadArguments(argc, argv);
  QyInitialize(argc, argv);

  if (QyBindAsync(SIMPLE_AUTH) != QyBindOk)
    exit(1);
  

  InitWidgets();
  InitTermPrint();
  InitHelp();
  Initialize();
  InitHistory();

  if (QyBindAsyncWait(TRUE) == QyBindOk)
    Interact();

  Quit("Exiting slu..\n", 0);
}

static void ReadArguments(argc, argv)
     unsigned int argc;
     char **argv;
{
  register char *cp;
  char **av;
  char buf[BUFLEN];

  if (argc <= 1) return;

  av = argv;
  av++;

  while ((cp = *av) && (*cp == '-'))
    {
      argc--;
      switch (*++cp)
	{
	case 't':

	  if (lexequ(cp, "-test") != 0)
	    {
	      testing=TRUE;
	      printf("Using hardcoded test setup.\n");
	    }
	  break;

/****************************************************************************/

	 case 'i':
         case 'I':
 
         /**************************************************
          *
          * Set ISODE directories.
          *
          */
 
         if (argc >= 2)
           {
	       av++;
               argc--;
	       cp=*av;
               if (*cp == '-') {
	               printf("You must specify a directory name after the");
		       printf(" -%c option which doesn't begin\n",*cp);
	               printf("with a '-' sign.\n");
		       exit(1);
			}
           }
         else {
               printf("You must specify a directory name after the");
	       printf(" -%c option\n",*cp);
	       exit(1);
	   }

	 (void) printf("Using ISODE directories under: %s\n",cp);

         (void) sprintf(buf, "%s/bin/", cp);
         (void) isodesetvar("binpath", strdup(buf), 1);

         (void) sprintf(buf, "%s/sbin/", cp);
         (void) isodesetvar("sbinpath", strdup(buf), 1);

         (void) sprintf(buf, "%s/etc/", cp);
         (void) isodesetvar("etcpath", strdup(buf), 1);
         break;

/****************************************************************************/

         case 's':

         /**************************************************
          *
          * Set location of "slurc" type file.
          *
          */
 
         if (argc >= 2)
           {
	       av++;
               argc--;
	       cp=*av;
               if (*cp == '-') {
	               printf("You must specify a file name after the");
		       printf(" -%c option which doesn't begin\n",*cp);
	               printf("with a '-' sign.\n");
		       exit(1);
			}
           }
         else {
               printf("You must specify a file name after the");
	       printf(" -%c option\n",*cp);
	       exit(1);
	      }

         (void) strcpy(config_file_name, cp);
	 (void) printf("Using user-taylor file: %s\n",config_file_name);

	 break;

	 case 'h':
		
printf("\nUsage: %s [flags]\n\n",argv[0]);
printf("Where the avaliable flags are:\n\n");
printf("-h\t\t- This help message\n\n");
printf("-s <filename>\t- filename of user tailor file to use\n\n");
printf("-i <directory>\t- ISODE directory to use.\n");
printf("-I <directory>\t- ISODE directory to use.\n\n");
printf("-u<username>\t- Username to use when binding to the directory\n");
printf("-p<password>\t- Password to use with username (see above)\n\n");
printf("-T<oidtable>\t- Oid table to use rather than the default\n\n");
printf("-c<DSA Address>\t- Connect to the given DSA\n\n");
printf("-test\t\t- Use some hardcoded options (INSTALLATION TEST ONLY!)\n\n");
printf("SLU version number: %s.%s (patchlevel %s) - %s\n", MAJOR_REVISION,
				MINOR_REVISION, PATCHLEVEL, RELEASEDATE);
		   exit(0);
		break;

	case 'v':
	case 'V':
		printf("SLU version number: %s.%s (patchlevel %s) - %s\n",
				MAJOR_REVISION,
				MINOR_REVISION,
				PATCHLEVEL,
				RELEASEDATE);
		exit(0);
		break;
	}


      av++;
    }
}

/*****************************************************************************
 * QuitCB()
 *
 * 
 *
 ****************************************************************************/

void QuitCB()
{
  Quit("", 0);
} /* QuitCB */

/*****************************************************************************
 * Quit()
 *
 * 
 *
 ****************************************************************************/

void Quit(error, sig)
     char    *error;
     int     sig;
{
  extern void EndWidgets();

  EndWidgets();
  (void) ds_unbind();
  hide_picture();
  (void) printf(error);
  exit(sig);
} /* Quit */

