/***************************************************************************
 *
 * SEARCH.C - UFN searching. 
 * 
 **************************************************************************/

/***************************************************************************
 *
 * @(#) search.c 1.2@(#)
 *
 ***************************************************************************/
 
#ifndef lint
static char *sccsid="@(#) search.c 1.2@(#)";
#endif


#include "search.h"
#include "dua.h"
#include "textdpy.h"
#include "entrylist.h"
#include "query.h"

/**************************************************
 *
 * Local functions.
 * 
 */

void Report();
static void Result();
int count_intermediate_results();

/*****************************************************************************
 * Search()
 *
 * Perform a UFN search.
 *
 ****************************************************************************/

void Search(name)
     char *name;
{
  QyErrorCode err;

  extern int selectentry_mode;

  ClearData();
  AddData("Starting search. Please Wait.....\n");
  AddData("[Or press ctrl-C to abort] \n\n");
  AddData("\n");
  Scrollbar('\0');

  selectentry_mode = NORMAL;

  err = QyUFNResolve(NULLEntryList, name, NULLStrCell, &req_id);
  switch (err)
    {
    case QERR_bad_value_syntax:
      ClearData();
      AddData("The directory name entered is invalid!");
      Scrollbar('\0');
      return;

    case QERR_internal_limit_reached:
      ClearData();
      AddData("Internal limit reached, cannot search!");
      Scrollbar('\0');
      return;
    }

  AddData("Searching, please wait.....\n");

  DirectoryWait(req_id, Result, Report);
} /* Search */

/*****************************************************************************
 * Report()
 *
 * A search status report has been received.
 *
 ****************************************************************************/

void Report(reqid)
     int reqid;
{
  char msg[LINESIZE];
  QyUFNReport report;
  entryList entrylist;
  int matches=0;

  report = QyGetUFNReport(reqid);

  switch (report->type)
    {
    case QyUFNmatches:

      matches=count_intermediate_results(report->srchlist);	
      (void) sprintf
	(msg, "Made %d %s for `%s'\n",matches,
				(matches==1) ? "match" : "matches",
				report->resolved);
	AddData(msg);

	for (entrylist=report->srchlist;entrylist!=NULL;
				entrylist=entrylist->next){
      		(void) sprintf
			(msg, "\t- %s\n",entrylist->sort_key);
		AddData(msg);
		}

	msg[0]='\0';
      
      break;

    case QyUFNattempt:
      
      /* Improve this message ! */
      (void) sprintf(msg, "Search restarted.\n");
      break;
    }

  AddData(msg);
  Scrollbar('\1');

  QyFreeUFNReport(&report);
} /* Report */

/*****************************************************************************
 * count_intermediate_results()
 *
 * 
 *
 ****************************************************************************/
int count_intermediate_results(int_report)
  entryList int_report;
{
int count=0;

	if (int_report->next != (entryList)NULL)
		count=count_intermediate_results(int_report->next);
	
	count++;
	return count;

} /* count_intermediate_results */

/*****************************************************************************
 * Result()
 *
 * 
 *
 ****************************************************************************/

static void Result(reqid)
     int reqid;
{
  QyUFNResult result;
  int errnum;
  QyErrorList errors;
  char msg[LINESIZE];
  char buff[BUFLEN];

  result = QyGetUFNResult(reqid);

  ClearData();
  AddData("Result received.\n");
  (void) strcpy(msg, "Search completed: ");

  /**************************************************
   *
   * Work out the error situation.
   * 
   */

  for (errnum = 0, errors = result->errors;
       errors != NULLError;
       errors = errors->next, errnum++)
    ;
  
  if (result->matchnum == 0)
    {
      if (errnum > 0)
	(void) strcat(msg, "Search failed in error!\n");
      else
	(void) strcat(msg, "Nothing found!\n");
      
      AddData(msg);
    }
  else if (result->matchnum == 1)
    {
      QyPrintFriendlyDN(result->matches->string_dn, buff);
      AddHistory("Searched for:",result->matches->string_dn);

      Show(result->matches->string_dn);
    }
  else if (result->matchnum > 1)
    {
      SetEntryList(result->matches);
      (void) ShowEntryList();
    }

  Scrollbar('\0');
  QyFreeUFNResult(&result);
} /* Result */





