/**************************************************************************
 *
 * SHOW.C - Read and display directory entries.
 * 
 **************************************************************************/

/***************************************************************************
 *
 * @(#) show.c 1.2@(#)
 *
 ***************************************************************************/
 
#ifndef lint
static char *sccsid="@(#) show.c 1.2@(#)";
#endif

#include <string.h>
#include "macros.h"
#include "query.h"
#include "message.h"
#include "dua.h"
#include "attrname.h"
#include "attrread.h"
#include "msg.h"
#include "show.h"

/**************************************************
 *
 * Local functions.
 * 
 */

static void Result();
static void DisplayEntry();

/**************************************************
 *
 * Local data.
 * 
 */

char CurrentEntry[LINESIZE]; /* Last read entry */

/*****************************************************************************
 * Show()
 *
 * Read and show the named (= DN) entry.
 *
 ****************************************************************************/

void Show(name)
     STRING name;
{
  QyErrorCode err;
  int reqid;
  
  extern stringCell DisplayAttributeList;

  ClearData();
  AddData("Reading entry, please wait...\n");
  AddData("[or press ctrl-C to abort]");
  Scrollbar('\0');

  err = QyReadEntry(name, &reqid, DisplayAttributeList);
  switch (err)
    {
    default:
      break;
    }

  (void) strcpy(CurrentEntry, name);
  DirectoryWait(reqid, Result, (RequestCallback) NULL);
} /* Show */

/*****************************************************************************
 * Result()
 *
 * Display returned result.
 *
 ****************************************************************************/

static void Result(reqid)
     int reqid;
{
  QyReadResult result;
  char buf[LINESIZE];
  STRING frdn;

  result = QyGetReadResult(reqid);
  AddData("Read result received.\n");

  if (result->errors != NULLError)
    {
      ClearData();
      AddData("Result received: read failed in error!");
    }
  else
    {
      QyPrintFriendlyDN(result->base_object, buf);
      frdn = index(buf, ','); /* #### Hack, doesn\'t cater for DNs with , */
      
      if (frdn != (STRING) NULL)
	*frdn = '\0';

      frdn = buf;
      SetMessage(frdn);

      DisplayEntry(result->entry);
    }
  
  Scrollbar('\0');
  QyFreeReadResult(&result);
} /* Result */

/*****************************************************************************
 * DisplayEntry()
 *
 * 
 *
 ****************************************************************************/

static void DisplayEntry(entry)
     QyAttrValList entry;
{
  QyAttrValList attr;
  stringCell value;
  char line[LINESIZE], buf[LINESIZE];
  STRING start, end;

  ClearData();

  for (attr = entry; attr != NULLAVList; attr = attr->next)
    {
      (void) GetAttributeAlias(str2AttrT(attr->attr_name->string), buf);
      (void) sprintf(line, "%s:\n", buf);
      AddData(line);

      for (value = attr->val_list; value != NULLStrCell; value = value->next)
	{
	  /**************************************************
	   *
	   * If attribute value is multi-lined add each
	   * separately.
	   * 
	   */

	  start = value->string;
	  while (TRUE)
	    {
	      end = index(start, '\n');
	      if (end == (STRING) NULL)
		break;

	      *end = '\0';

	      (void) sprintf(buf, "%s\n", start);
	      start = TidyString(buf);

	      (void) sprintf(line, "\t%s\n", start);
	      AddData(line);

	      *end = '\n';
	      start = ++end;
	    }

	  (void) sprintf(buf, "\t%s\n", start);
	  start = TidyString(buf);

	  (void) sprintf(line, "\t%s\n", start);
	  AddData(line);
	}

      AddData("\n");
    }

  /* (void) DisplayFilter(); */

  Scrollbar('\0');
} /* DisplayEntry */

/*****************************************************************************
 * ShowOwner()
 *
 * Read and display owner of current entry.
 *
 ****************************************************************************/

void ShowOwner()
{
  STRING endptr;

  if (isempty(CurrentEntry))
    return;

  endptr = rindex(CurrentEntry, '@');
  if (endptr == (STRING) NULL)
    return;

  *endptr = '\0';

  AddHistory("Owner Entry:",CurrentEntry); 

  Show(CurrentEntry);
  /* SetMessage("Owner entry displayed."); */
} /* ShowOwner */
