/* widget.c - Provides the screen and widget code */

/***************************************************************************
 *
 * @(#) widget.c 1.2@(#)
 *
 ***************************************************************************/
 
#ifndef lint
static char *sccsid="@(#) widget.c 1.2@(#)";
#endif

/*
 * This file has been modified by Damanjit Mahl @ Brunel University
 * and Paul Wain @ Brunel University
 */

/*
 *				  NOTICE
 *
 *    Acquisition, use, and distribution of this module and related
 *    materials are subject to the restrictions of a license agreement.
 *    Consult the Preface in the Users Manual for the full terms of
 *    this agreement.
 *
 */

/*****************************************************************************/
/* File:	widget.c
/* Author:	Paul Sharpe @ GEC Research, Hirst Research Centre.
/* Date:	August 12, 1988.
/* Function:	Provides the screen and widget code.
/*
/* DISCLAIMER:
/* This source file is deemed to be public domain: any person may use the
/* code in any way, on two simple provisos -
/*   1 - That this header remains in this file,
/*   2 - It is accepted that neither the author, nor the authors employees
/*       accept any responsibility for the use, abuse or misuse of the
/*       contained code.
/* No guarantee is made by the author to provide maintainance or up-grading
/* of this source. However, any adaptations made to this file will be viewed
/* at least with interest.
/*****************************************************************************/

#define __time_h

#include <strings.h>
#include <ctype.h>
#include <stdio.h>
#include <setjmp.h>
#include <signal.h>

#include "types.h"
#include "widget.h"
#include "sequence.h"

#include "search.h"
#include "browse.h"
#include "show.h"
#include "history.h"
#include "main.h"
#include "entrylist.h"
#include "help.h"
#include "termprint.h"
#include "dua.h"

/****************************************************************
 *
 * Local data.
 * 
 */

#define STRINGLEN 512
static char SearchValue[STRINGLEN];
static char GotoValue[STRINGLEN];
static unsigned int ScrollPos = 1;
static char CommandChar; /* Widget activation key-char */
int DataHeight;

void Do_refresh_number();

/****************************************************************
 *
 * Widget lists.
 * 
 */

/* REMEMBER TO NULL-TERMINATE ALL OF THESE STRUCTURE ARRAYS     */
/*--------------------------------------------------------------*/

WIDGET mainwdgts[] =
{
  {COMMAND, "q Quit ",	 	'q', QuitCB,   0,0,0,0},
  {COMMAND, "? Help ",	 	'?', HelpMain,   0,0,0,0},
  {COMMAND, "b Browse ",	'b', Browse,   0,0,0,0},
  {COMMAND, "o View owner",     'o', ShowOwner, 0, 0, 0, 0},
  {COMMAND, "h History ", 	'h', ShowHistory, 0,0,0,0},
  {DIALOG,  "Go To Number:",	'*', GotoEntry, 0,0,EXPAND,0},
  {DIALOG , "Name: ",		'n', Search,     0,CRNL,EXPAND,0},
  {LABEL,   " SLU - Directory Services. ", CENTRE,NULLFN,CRNL,0,EXPAND,0},
  /* Scrollbar always last */
  {SCROLLBAR, "",               '%', NULL, 0,0,3,0},
  {FINISH,  "", 0,NULLFN}
};

/*--------------------------------------------------------------*/

struct active
{
  int count;
  WIDGET *widgets[MAXACTIVE];
  char lastindex[MAXACTIVE];
  WINDOW *text[MAXACTIVE];
} activelist;

WINDOW *DataWindow;
WIDGET *currwidgets;
jmp_buf	env;

/*****************************************************************************
 * InitWidgets()
 *
 * Initialize curses stuff and display main widgets.
 *
 ****************************************************************************/

void InitWidgets()
{
  (void) initscr();
  (void) noecho();
  (void) crmode();
  DataWindow = stdscr;
  activelist.count = 0;

  SetDisplayWidgets(mainwdgts, -1);
  SetDialogString(GetWidget(mainwdgts, 'n'), SearchValue, STRINGLEN);
  SetDialogString(GetWidget(mainwdgts, '*'), GotoValue, STRINGLEN);

} /* InitWidgets */

/*****************************************************************************
 * RefreshWindow()
 *
 * Bring WINDOW changes up to date.
 *
 ****************************************************************************/

void RefreshWindow(win)
     WINDOW *win;
{
  (void) wrefresh(win);
} /* RefreshWindow */

/*****************************************************************************
 * SetDisplayWidgets()
 *
 * Chnage the currently active set of widgets.
 *
 ****************************************************************************/

void SetDisplayWidgets(thesewdgts, y)
     WIDGET *thesewdgts;
     int y;
{
  currwidgets = thesewdgts;
  lowy = PosWidgets(thesewdgts, y);
  DataWindow = newwin(LINES-1-lowy, COLS-3, lowy, 3);
  DataHeight = LINES-1-lowy;
  refresh();
  (void) scrollok(DataWindow, FALSE);
  (void) wrefresh(DataWindow);
  MakeWidgets(thesewdgts);
  ActiveWidget(thesewdgts, DataWindow);
  RefreshWidgets(thesewdgts);
} /* SetDisplayWidgets */

/*****************************************************************************
 * LineNumber()
 *
 * Return the number of lines in the screen.
 *
 ****************************************************************************/

int LineNumber()
{
  return (int) LINES;
} /* LineNumber */

/*****************************************************************************
 * GetYPos()
 *
 * Get current Y position of window.
 *
 ****************************************************************************/

int GetYPos()
{
  int y, x;
  getyx(DataWindow, y, x);

  /* Get rid of lint warning */
  x = x + 1;
  return y + lowy;
} /* GetYPos */

/*****************************************************************************
 * PosWidgets()
 *
 * Determine the positions of the widgets: return the lowest point.
 *
 ****************************************************************************/

int PosWidgets(thesewdgts, starty)
     int	starty;
     WIDGET	thesewdgts[];

{
  register int	cnt = 0, x = 0, hght = WDGTHGHT;
  
  /* If no explicit position provided, put on the next level */
  if (starty < 0)
    starty = LowestY();
  
  /* Set the position of the widgets, as dynamically as possible */
  while (thesewdgts[cnt].type != FINISH)
    {
      if (thesewdgts[cnt].type != DUMMY)
	{
	  /* If the widget is a scrollbar put on a new line */
	  if (thesewdgts[cnt].type == SCROLLBAR)
	    {
	      starty += WDGTHGHT;
	      hght = LINES - starty;
	      x = 0;
	    }
	  else if (thesewdgts[cnt].y == CRNL)
	    /* If the initial structure y-value is CRNL, put on a new line */
	    {
	      starty += WDGTHGHT;
	      hght = WDGTHGHT;
	      x = 0;
	    }
	  
	  /**************************************************
	   *
	   * If we ain\'t got a width, make one based
	   * on the type/label-length.
	   * 
	   */
	  
	  if (thesewdgts[cnt].wdth <= 0)
	    SetWidgetWidth(&thesewdgts[cnt], x);
	  
	  /* If the widget won\'t fit, start a new line of widgets */
	  if (x + thesewdgts[cnt].wdth > COLS)
	    {
	      starty += WDGTHGHT;
	      x =  0;
	    }
	  
	  thesewdgts[cnt].x = x;
	  thesewdgts[cnt].y = starty;
	  thesewdgts[cnt].hght = hght;

	  x += thesewdgts[cnt].wdth;
	  ++cnt;
	}
      else 
	++cnt;
    }

  return starty;
} /* PosWidgets */

/*****************************************************************************
 * MakeWidgets()
 *
 * Create the widgets.
 *
 ****************************************************************************/

void MakeWidgets(wdgts)
     WIDGET wdgts[];
{
  register int cnt = 0;
  register WIDGET *wdgt;

  /* Now try to make, box and label the widget windows */
  while (wdgts[cnt].type != FINISH)
    {
      if (wdgts[cnt].type != DUMMY)
	{
	  wdgt = &wdgts[cnt++];
	  wdgt->wndw = newwin(wdgt->hght,wdgt->wdth,wdgt->y,wdgt->x);
	  BoxWidget(wdgt, '-', '|');
	  PrintWidget(wdgt);
	}
      else
	++cnt;
    }
} /* MakeWidgets */

/*****************************************************************************
 * SetWidgetWidth()
 *
 * Set a widgets width, based on the TYPE of widget and label length.
 *
 ****************************************************************************/

void SetWidgetWidth(wdgt, currx)
     int currx;
     WIDGET *wdgt;
{
  char expand;
  register int len = -1, cnt = -1;

  if (expand = (wdgt->wdth == EXPAND))
    wdgt->wdth = COLS - currx;
  
  switch (wdgt->type)
    {
    case LABEL:

      len = strlen(wdgt->label) + 2;
      if (wdgt->wdth < len)
	{
	  if (expand)
	    wdgt->wdth = COLS;
	  else
	    wdgt->wdth = len;
	}
      
      break;
      
    case DIALOG:

      len = strlen(wdgt->label) + 2;
      if (wdgt->dstrlen > DIALOGLEN)
	len += DIALOGLEN;
      else
	len += wdgt->dstrlen;
      
      if (wdgt->wdth < len)
	{
	  if (expand)
	    wdgt->wdth = COLS;
	  else
	    wdgt->wdth = len;
	}
      
      break;
      
    case TOGGLE:

      if (wdgt->tvalues == (char **)NULL)
	wdgt->wdth = strlen(wdgt->label) + 2;
      else
	{
	  while(wdgt->tvalues[++cnt] != (char *)NULL)
	    if (strlen(wdgt->tvalues[cnt]) > len)
	      len = strlen(wdgt->tvalues[cnt]);
	  wdgt->wdth = strlen(wdgt->label) + len + 2;
	}
      
      break;
      
    default:

      wdgt->wdth = strlen(wdgt->label) + 2;
      break;
    }
} /* SetWidgetWidth */

/*****************************************************************************
 * KillWidgets()
 *
 * Erase and remove the widgets, decrementing the activelist counter.
 *
 ****************************************************************************/

void KillWidgets(thesewdgts)
     WIDGET *thesewdgts;
{
  register int cnt = 0;
  
  while (thesewdgts[cnt].type != FINISH)
    if (thesewdgts[cnt].type != DUMMY)
      {
	(void) wclear(thesewdgts[cnt].wndw);
	(void) wrefresh(thesewdgts[cnt++].wndw);
      }
    else
      ++cnt;

  cnt = 0;
  (void) wclear(DataWindow);
  (void) wrefresh(DataWindow);

  while (thesewdgts[cnt].type != FINISH)
    if (thesewdgts[cnt].type != DUMMY)
      delwin(thesewdgts[cnt++].wndw);
    else
      ++cnt;

  delwin(DataWindow);
  DeleteActiveWidget();
  
  DataWindow = activelist.text[activelist.count-1];
  if (DataWindow != (WINDOW *)NULL)
    (void) wrefresh(DataWindow);
} /* KillWidgets */

/* THESE FUNCTIONS MANIPULATE THE ACTIVELIST ARRAY OF WIDGETS */

/*****************************************************************************
 * ActiveWidget()
 *
 * This should check that the number of active widgets is not excessive,
 * or have the activelist really as a linked list.
 *
 ****************************************************************************/

/* ARGSUSED */
void ActiveWidget(wdgts, text)
     WIDGET	wdgts[];
     WINDOW	*text;
{
  activelist.widgets[activelist.count] = wdgts;
  activelist.text[activelist.count] = DataWindow;
  ++(activelist.count);
} /* ActiveWidget */

/*****************************************************************************
 * DeleteActiveWidget()
 *
 * 
 *
 ****************************************************************************/

void DeleteActiveWidget()
{
  if (activelist.count > 0)
    --(activelist.count);
} /* DeleteActiveWidget */

/*****************************************************************************
 * ActiveIndex()
 *
 * 
 *
 ****************************************************************************/

void ActiveIndex(indx)
     int indx;
{
  activelist.lastindex[activelist.count - 1] = indx;
} /* ActiveIndex */

/*****************************************************************************
 * Redraw()
 *
 * Refresh each of the active widgets and the current DataWindow window.
 * 
 ****************************************************************************/

void Redraw()
{
  register int i;

  clearok(curscr, TRUE); 
  for (i = 0; i < activelist.count; i++) {
    RefreshWidgets(activelist.widgets[i]);
    }

  (void) wrefresh(DataWindow);
} /* Redraw */

/*****************************************************************************
 * RefreshWidgets()
 *
 * 
 *
 ****************************************************************************/

void RefreshWidgets(thesewdgts)
     WIDGET *thesewdgts;
{
  int i = 0;

  while(thesewdgts[i].wndw != (WINDOW *) NULL && thesewdgts[i].type != DUMMY)
    {
      touchwin(thesewdgts[i].wndw);
      (void) wrefresh(thesewdgts[i++].wndw);
    }
} /* RefreshWidgets */

/*****************************************************************************
 * BoxWidget()
 *
 * Draw a perimeter box around WDGT, with horizontal char XCH, etc.
 *
 ****************************************************************************/

void BoxWidget(wdgt, xch, ych)
     char xch, ych;
     register WIDGET *wdgt;
{
  register int x, y;

  mvwaddch(wdgt->wndw, 0, 0, '.');

  for (x = 1; x < wdgt->wdth-1; x++)
    (void) waddch(wdgt->wndw, xch);

  (void) waddch(wdgt->wndw, '.');
  
  mvwaddch(wdgt->wndw, 1, 0, ych);
  for (y = 1; y < wdgt->hght-1; y++)
    {
      (void) mvwaddch(wdgt->wndw, y, 0, ych);
      (void) mvwaddch(wdgt->wndw, y, wdgt->wdth-1, ych);
    }

  mvwaddch(wdgt->wndw, wdgt->hght-1, 0, '`');
  for (x = 1; x < wdgt->wdth-1; x++)
    (void) waddch(wdgt->wndw, xch);

  (void) waddch(wdgt->wndw, '\'');
} /* BoxWidget */

/* THESE ROUTINES PRINT THE INDIVIDUAL WIDGET BOXES */

/*****************************************************************************
 * PrintWidget()
 *
 * Print a widgets label, dependant on the widget type,
 *
 ****************************************************************************/

void PrintWidget(wdgt)
     WIDGET *wdgt;
{
  switch(wdgt->type)
    {
    case LABEL:
      PrintLabel(wdgt);
      break;
      
    case DIALOG:
      PrintDialog(wdgt);
      break;
      
    case TOGGLE:
      PrintToggle(wdgt);
      break;
      
    default :
      PrintCommand(wdgt);
      break;
    }
} /* PrintWidget */

/*****************************************************************************
 * PrintScrollbar()
 *
 * 
 *
 ****************************************************************************/

void PrintScrollbar(list_size, first, display_num)
     int list_size, first, display_num;
{
  WIDGET *wdgt;
  int cnt, bar_size, bar_pos = 0, space_size;

  for (wdgt = &currwidgets[0];
       wdgt->type != SCROLLBAR && wdgt->type != FINISH;
       wdgt++)
    ;

  if (wdgt->type == FINISH)
    return;
  
  (void) wclear(wdgt->wndw);
  BoxWidget(wdgt, '-', '|');
  
  if (list_size == 0 || display_num == 0)
    return;

  space_size = wdgt->hght - 4;
  if (display_num == list_size)
    {
      bar_size = space_size;
      bar_pos = 1;
    }
  else
    {
      bar_size = (display_num*space_size)/(list_size+1);
      bar_size = bar_size? bar_size: 1;
      
      while(!((list_size*bar_pos)/(first*(space_size+1))))
	bar_pos++;
    }
  
  while((bar_size + bar_pos - 1) > space_size)
    {
      bar_size--;
    }
  
  for (cnt = 0; cnt < bar_size; cnt++)
    (void) mvwaddch(wdgt->wndw, cnt+1+bar_pos, 1, '*');
  
  (void) mvwaddch(wdgt->wndw, 1, 1, '[');
  (void) mvwaddch(wdgt->wndw, wdgt->hght-2, 1, ']');
  
  (void) wrefresh(wdgt->wndw);
} /* PrintScrollbar */

/*****************************************************************************
 * PrintLabel()
 *
 * Print a LABEL widgets label string, dependant on the justification char.
 *
 ****************************************************************************/

void PrintLabel(wdgt)
     WIDGET *wdgt;
{
  register int x, labellen, wdgtlen;

  labellen = strlen(wdgt->label);
  wdgtlen = wdgt->wdth - 2;
  
  if (labellen > wdgtlen)
    wdgt->label[wdgtlen] = '\0';
  
  if (wdgt->callch & CENTRE)
    x = (wdgtlen - labellen)/2;
  else if (wdgt->callch & LEFT)
    x = 0;
  else if (wdgt->callch & RIGHT)
    x = wdgtlen - labellen;
  
  (void) wclear(wdgt->wndw);
  BoxWidget(wdgt, '-', '|');

  mvwaddstr(wdgt->wndw,1,1+x,wdgt->label);
  (void) wrefresh(wdgt->wndw);
} /* PrintLabel */

/*****************************************************************************
 * PrintDialog()
 *
 * Print a DIALOG widget label: if it don\'t all fit, show the last part.
 *
 ****************************************************************************/

void PrintDialog(wdgt)
     WIDGET *wdgt;
{
  register int	length, maxlen;
  register char	*showptr;

  PrintLabel(wdgt);

  if (wdgt->dstr != (char *) NULL)
    {
      length = strlen(wdgt->dstr);
      maxlen = wdgt->wdth - 4 - strlen(wdgt->label);

      if (length > maxlen)
	showptr = &(wdgt->dstr[length - maxlen]);
      else
	showptr = wdgt->dstr;

      (void) mvwprintw(wdgt->wndw,
		       1,
		       1,
		       "%s%c%s",
		       wdgt->label, 
		       (length > maxlen)?'<':' ',
		       showptr);
    }

  (void) wrefresh(wdgt->wndw);
} /* PrintDialog */

/*****************************************************************************
 * PrintToggle()
 *
 * Print a TOGGLE widget label, and the current toggle value.
 *
 ****************************************************************************/

void PrintToggle(wdgt)
     WIDGET *wdgt;
{
  (void) wclear(wdgt->wndw);
  BoxWidget(wdgt, '-', '|');

  if (wdgt->tvalues == (char **) NULL)
    return;

  mvwaddstr(wdgt->wndw,1,1,wdgt->label);
  (void) waddstr(wdgt->wndw,wdgt->tvalues[wdgt->tindx]);
  (void) wclrtoeol(wdgt->wndw);
  (void) mvwaddch(wdgt->wndw,1,wdgt->wdth-1,'|');
  (void) wrefresh(wdgt->wndw);
} /* PrintToggle */

/*****************************************************************************
 * PrintCommand()
 *
 * Print a COMMAND widget label.
 *
 ****************************************************************************/

void PrintCommand(wdgt)
     WIDGET *wdgt;
{
  mvwaddstr(wdgt->wndw,1,1,wdgt->label);
  (void) wrefresh(wdgt->wndw);
} /* PrintCommand */

/* THESE ROUTINES GET AND REACT TO A USERS INPUT FROM THE KEYBOARD */

/*****************************************************************************
 * Interact()
 *
 * Loop forever, calling widget callback functions when activated.
 *
 ****************************************************************************/

void Interact()
{
  char ch;
  int indx;

  for (;;)
    {
      /* Get a character input, and set the interrupt jump vector */
      (void) setjmp(env);
      (void) wrefresh(DataWindow);
      refresh();
      ch = (char) getchar();
      if (isupper(ch))
	ch = tolower(ch);

      /* Allow the user to refresh the entire screen, with a CTRL-L */
      if (ch == '\014')
	{
	  Do_refresh_number(); /* Hack to ensure number entry works! */
	  Redraw();
	  Scrollbar('\0');
	  continue;
	}

      /* Search through the current widgets for one matching that required */
      CommandChar = ch;
      indx = FindActiveInput(ch);

      if (indx >= 0) {
        (void) signal(SIGINT,Do_Interupt);
	CallCallback(indx);
	}

      ch = 0;
    }
} /* Interact */

/*****************************************************************************
 * FindActiveInput()
 *
 * Find a callback \'ch\' from the currently active set of widgets.
 *
 ****************************************************************************/

int FindActiveInput(ch)
     char ch;
{
  register int indx;
  register WIDGET *wdgts;

  if (ch > 'z' || ch < 'a')
    {
      switch (ch)
	{
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	  ch = '*';
	  break;

	case '[':
	case ']':
	  ch = '%';
	  break;
	}
    }

  /* See whether the \'ch\' exists in the currently active widgets */
  wdgts = activelist.widgets[activelist.count - 1];
  indx = GetWidgetIndex(wdgts, ch);

  if (indx >= 0)
    return indx;
  
  /* If not, check the previously active widgets, if possible */
  if (activelist.count <= 1)
    return -1;

  indx = GetWidgetIndex(activelist.widgets[activelist.count - 2], ch);
  if (indx >= 0)
    {
      KillWidgets(activelist.widgets[activelist.count - 1]);
      return indx;
    }

  return -1;
} /* FindActiveInput */

/*****************************************************************************
 * CallCallback()
 *
 * 
 *
 ****************************************************************************/

void CallCallback(indx)
     int indx;
{
  WIDGET *wdgts;

  ActiveIndex(indx);

  wdgts = activelist.widgets[activelist.count - 1];
  switch (wdgts[indx].type)
    {
    case DIALOG:
      Dialog(&wdgts[indx]);
      (*wdgts[indx].callfn)(wdgts[indx].dstr);
      break;

    case TOGGLE:
      Toggle(&wdgts[indx]);
      (*wdgts[indx].callfn)();
      break;

    case SCROLLBAR :
      Scrollbar(CommandChar);
      break;
      
    default:
      (*wdgts[indx].callfn)();
      break;
    }
} /* CallCallback */

/* THESE ROUTINES SEARCH THE ACTIVE WIDGET SET FOR ONE SPECIFIED WIDGET */

/*****************************************************************************
 * GetWidget()
 *
 * Find a widget based on the call-back character.
 *
 ****************************************************************************/

WIDGET *GetWidget(wdgts, callch)
     char callch;
     WIDGET wdgts[];
{
  register int indx;

  indx = GetWidgetIndex(wdgts, callch);
  if (indx >= 0)
    return &(wdgts[indx]);

  return (WIDGET *) NULL;
} /* GetWidget */

/*****************************************************************************
 * GetWidgetIndex()
 *
 * 
 *
 ***************************************************************************/

int GetWidgetIndex(wdgts, callch)
     char callch;
     WIDGET wdgts[];
{
  register int cnt = 0;

  while (wdgts[cnt].type != FINISH)
    {
      if (callch == wdgts[cnt].callch)
	break;

      ++cnt;
    }

  if (wdgts[cnt].type != FINISH)
    return cnt;

  return -1;
} /* GetWidgetIndex */

/* THESE ROUTINES MANIPULATE THE DIALOG WIDGETS */

/*****************************************************************************
 * Dialog()
 *
 * 
 *
 ****************************************************************************/

void Dialog(wdgt)
     WIDGET *wdgt;
{
  register int i, length = 0, labellen, maxlen;
  register char ch, *endptr, *showptr;
  register char	*blanks;
  extern char *malloc();
  
  boldon();

  labellen = strlen(wdgt->label);	/* The length of the prompt string  */

  /* Hack for widget that reads numeric strings */
  if (wdgt->callch == '*')
    {
     *wdgt->dstr = CommandChar;
     wdgt->dstr[1]='\0';
     /* (void) mvwprintw(wdgt->wndw, 1, 1, "%s %s", wdgt->label, wdgt->dstr);
*/
    }


  if (wdgt->dstr != NULLSTRING)
    length = strlen(wdgt->dstr);	/* The length of the current string */
  
  maxlen = wdgt->wdth - 4 - labellen;	/* The maximum length of shown str  */
  blanks = malloc((unsigned)(maxlen + 2));

  for (i = 0; i < maxlen; i++)
    blanks[i] = ' ';

  blanks[i] = '\0';
  
  endptr = &(wdgt->dstr[length]);	/* The next character pos\'n to fill */
  *endptr = '\0';
  
  if (length > maxlen)
    showptr = &(wdgt->dstr[length - maxlen]);
  else
    showptr = wdgt->dstr;

  (void) mvwprintw(wdgt->wndw, 1, 1, "%s%c%s", wdgt->label,
		   (length > maxlen)? '<' : ' ', 
		   showptr);

  (void) wrefresh(wdgt->wndw);
  
  while ((ch = getchar() & 127) != '\n' && ch != '\r' && ch != '\f')
    {
      if (ch == '\014')
	{ /* Allow for redrawing */
	  Do_refresh_number(); /* Hack to ensure number entry works! */
	  Redraw();
	  continue;
	}
      
      if (ch == '\b' || ch == 127)
	{ /* Delete a character, with wrapping */
	  if (length == 0)
	    continue;

	  *(--endptr) = '\0';	    /* Make the last character NULL */
	  if (showptr > wdgt->dstr) /* We have parts of the string hidden */
	    --showptr;

	  --length;
	  if (length < maxlen)
	    {	/* Only need to erase one character */
	      (void) waddstr(wdgt->wndw,"\b \b");
	      (void) wrefresh(wdgt->wndw);
	      continue;
	    }

	  /* We'll have to erase everything */
	  (void) wprintw(wdgt->wndw, "\r|%s%c%s \b", wdgt->label,
			 (length <= maxlen)? ' ' : '<',
			 showptr);

	  (void) wrefresh(wdgt->wndw);
	  continue;
	}
      
      if (ch == 21)
	{ /* ^U to delete the entire line of text */
	  length = 0;
	  endptr = wdgt->dstr;
	  *endptr = '\0';
	  showptr = wdgt->dstr;

	  (void) wprintw(wdgt->wndw, "\r|%s %s\r|%s ", wdgt->label, blanks,
			 wdgt->label);
	  (void) wrefresh(wdgt->wndw);

	  continue;
	}
      
      /**************************************************
       *
       * Otherwise, add the character if there is
       * room and it ain't a control code
       * 
       */

      if (length == 1024 || ch < 32)
	continue;

      if (length == wdgt->dstrlen)
	{
	  *endptr++ = ch;
	  *endptr = '\0';
	  SetDialogString(wdgt, showptr, wdgt->dstrlen);
	  continue;
	}

      *endptr++ = ch;
      *endptr = '\0';

      if (++length <= maxlen)
	{ /* Just add this character to the end */
	  (void) waddch(wdgt->wndw, ch);
	  (void) wrefresh(wdgt->wndw);
	  continue;
	}

      ++showptr;
      (void) wprintw(wdgt->wndw, "\r|%s<%s", wdgt->label, showptr);
      (void) wrefresh(wdgt->wndw);

    }

  boldoff();

  free(blanks);
} /* Dialog */

/*****************************************************************************
 * SetDialogString()
 *
 * 
 *
 ****************************************************************************/

void SetDialogString(wdgt, dstr, maxlen)
     char *dstr;
     WIDGET *wdgt;
     int maxlen;
{
  if (wdgt->type != DIALOG)
    return;

  wdgt->dstr = dstr;
  wdgt->dstrlen = maxlen;
} /* SetDialogString */

/*****************************************************************************
 * GetDialogText()
 *
 * 
 *
 ****************************************************************************/

int GetDialogText(wdgt, str)		/* 'str' must be long enough... */
     char str[];
     WIDGET *wdgt;
{
  if (wdgt->type != DIALOG || wdgt->dstr == (char *) NULL)
    return FALSE;

  (void) strcpy(str, wdgt->dstr);
  return TRUE;
} /* GetDialogText */

/* THESE ROUTINES MANIPULATE THE TOGGLE WIDGETS */

/*****************************************************************************
 * Toggle()
 *
 * 
 *
 ****************************************************************************/

void Toggle(wdgt)
     WIDGET *wdgt;
{
/*  WIDGET *vwdgt; 
  int av_indx; */

  if (wdgt->tvalues == (char **) NULL)
    return;

/*
  if (wdgt == GetWidget(currwidgets, 't'))
    {
      typetoggled = 1;
      vwdgt = GetWidget(currwidgets, 's');
      (void) strcpy(filtvalue[wdgt->tindx], vwdgt->dstr);

      av_indx = 0;
      while (av_typeindx[av_indx] != wdgt->tindx && av_typeindx[av_indx] >= 0) 
	av_indx++;

      if (av_typeindx[av_indx] == wdgt->tindx)
	av_indx++;
      
      if (av_typeindx[av_indx] < 0)
	av_indx = 0;

      wdgt->tindx = av_typeindx[av_indx];
      
      (void) strcpy(vwdgt->dstr, filtvalue[wdgt->tindx]);
      typeindx = wdgt->tindx;
      
      printdialog(vwdgt);
    }
*/
  
  PrintToggle(wdgt);
} /* Toggle */

/*****************************************************************************
 * Scrollbar()
 *
 * 
 *
 ****************************************************************************/

void Scrollbar(comm)
     char comm;
{
  int lines, count;
  extern stringCell DataList;

  lines = SeqCount(DataList);

  switch (comm)
    {
    case ']':

      count = ScrollPos + DataHeight;
      break;

    case '[':

      count = ScrollPos - DataHeight;
      break;

    case '\0':
      
      count = 1;
      break;
    }

  if (count > (lines - DataHeight))
    count = lines - DataHeight;
  
  if (count < 1)
    count = 1;

  ScrollPos = count;
  ShowData(ScrollPos - 1);

  PrintScrollbar(lines, ScrollPos, DataHeight);
} /* Scrollbar */

/*****************************************************************************
 * ResetScrollbar()
 *
 * 
 *
 ****************************************************************************/

void ResetScrollbar()
{
  ScrollPos = 1;
} /* ResetScrollbar */

/*****************************************************************************
 * SetToggleString()
 *
 * 
 *
 ****************************************************************************/

void SetToggleString(wdgt, togglstrs, togglindx)
     int togglindx;
     char **togglstrs;
     WIDGET *wdgt;
{
  if (wdgt->type != TOGGLE)
    return;

  wdgt->tvalues = togglstrs;
  wdgt->tindx = togglindx;
} /* SetToggleString */

/*****************************************************************************
 * SetToggleIndex()
 *
 * 
 *
 ****************************************************************************/

int SetToggleIndex(wdgt, indx)
     int indx;
     WIDGET *wdgt;
{
  int i;
  
  if (wdgt->type != TOGGLE || wdgt->tvalues == (char **) NULL)
    return FALSE;

  for (i = 0; i < indx; i++)
    if (wdgt->tvalues[i] == (char *)NULL)
      break;
  
  if (i != indx) /* There ain\'t that many toggle strings */
    return FALSE;

  wdgt->tindx = indx;
  return TRUE;
} /* SetToggleIndex */

/*****************************************************************************
 * GetToggleIndex()
 *
 * 
 *
 ****************************************************************************/

int GetToggleIndex(wdgt)
     WIDGET *wdgt;
{
  if (wdgt->type != TOGGLE || wdgt->tvalues == (char **)NULL)
    return -1;

  return wdgt->tindx;
} /* GetToggleIndex */

/*****************************************************************************
 * GetToggleString()
 *
 * 
 *
 ****************************************************************************/

int GetToggleString(wdgt, str) /* \'str\' must be long enough... */
     WIDGET *wdgt;
     char str[];
{
  if (wdgt->type != TOGGLE || wdgt->tvalues == (char **) NULL)
    return FALSE;

  (void) strcpy(str, wdgt->tvalues[wdgt->tindx]);
  return TRUE;
} /* GetToggleString */

/* THESE ROUTINES MANIPULATE THE LABEL WIDGETS */

/*****************************************************************************
 * SetLabel()
 *
 * 
 *
 ****************************************************************************/

void SetLabel(wdgt, label)
     WIDGET *wdgt;
     char *label;
{
  wdgt->label = label;
} /* SetLabel */

/* MISCELLANEOUS FUNCTIONS */

/*****************************************************************************
 * LowestY()
 *
 * Try to locate the bottom of the last set of widgets displayed.
 *
 ****************************************************************************/

int LowestY()
{
  register int cnt = 0;
  register WIDGET *wdgts;
  
  if (activelist.count <= 0)
    return 0;

  wdgts = activelist.widgets[activelist.count - 1];
  while (wdgts[cnt].type != FINISH)
    ++cnt;

  if (cnt == 0)
    return 0;

  return (wdgts[cnt-1].y) + WDGTHGHT;
} /* LowestY */

/*****************************************************************************
 * wprint()
 *
 * This satisfies the generalised printing structure.
 *
 ****************************************************************************/

/* ARGSUSED */
void wprint(here, fmt, a,b,c,d,e,f,g,h,i,j)
     WINDOW *here;
     char *fmt, *a,*b,*c,*d,*e,*f,*g,*h,*i,*j;
{
  (void) wprintw(here,fmt,a,b,c,d,e,f,g,h,i,j);
  (void) wrefresh(here);
} /* wprint */

/*****************************************************************************
 * tprint()
 *
 * This can be called as a way for an application to print text.
 * 
 ****************************************************************************/

/* VARARGS */
void tprint(fmt, a,b,c,d,e,f,g,h,i,j)
     char *fmt, *a,*b,*c,*d,*e,*f,*g,*h,*i,*j;
{
  (void) wprintw(DataWindow,fmt,a,b,c,d,e,f,g,h,i,j);
  (void) wrefresh(DataWindow);
} /* tprint */

/*****************************************************************************
 * xprint()
 *
 * 
 *
 ****************************************************************************/

void xprint(fmt)
     char *fmt;
{
  (void) wprintw(DataWindow, "%s", fmt);
} /* xprint */

/*****************************************************************************
 * xprintint()
 *
 * 
 *
 ****************************************************************************/

void xprintint(fmt, a)
     char *fmt;
     int a;
{
  (void) wprintw(DataWindow,fmt, a);
} /* xprintint */

/*****************************************************************************
 * ClearWindow()
 *
 * 
 *
 ****************************************************************************/

void ClearWindow(win)
     WINDOW *win;
{
  clearok(win, TRUE);
  (void) wclear(win);
} /* ClearWindow */

/*****************************************************************************
 * jumpback()
 *
 * Jump back to the interact function only on an interrupt.
 *
 ****************************************************************************/

void jumpback()
{
  (void) waddstr(DataWindow,"\n*** Interrupted ***\n");
  (void) wrefresh(DataWindow);
  longjmp(env, TRUE);
} /* jumpback */

/* This is used as a declaration, when no function callback is required */
void nullfn()
{}

/*****************************************************************************
 * QuitWidgets()
 *
 * This is used by widgets that just want to kill the current level.
 *
 ****************************************************************************/

void QuitWidgets()
{
  (void) wclear(DataWindow);
  (void) wrefresh(DataWindow);
  KillWidgets(activelist.widgets[activelist.count - 1]);
} /* QuitWidgets */

/*****************************************************************************
 * EndWidgets()
 *
 * 
 *
 ****************************************************************************/

void EndWidgets()
{
  move(LINES-1, 0);
  refresh();
  endwin();
} /* EndWidgets */
 
/*****************************************************************************
 * Clear_goto_number()
 *
 * 
 *
 ****************************************************************************/

void Clear_goto_number()
{
int indx;
WIDGET *wdgt;

     indx = FindActiveInput('*');
     wdgt = &mainwdgts[indx];

     /* ClearWindow(wdgt->wndw); */

     (void) wclear(wdgt->wndw);
     BoxWidget(wdgt, '-', '|');

}

/*****************************************************************************
 * Do_interupt()
 *
 * Interupt handler for in a command.
 *
 ****************************************************************************/

 void Do_Interupt()
 {
 char input;

   ClearData();
   AddData("Command Aborted.\n");
   AddData("\n");
   AddData("Aborting request... hang on a second!\n");
   AddData("\n");
   Scrollbar('\0');
   QyAbortRequest(req_id);
   longjmp(env,0);
}

/*****************************************************************************
 * Do_Quit()
 *
 * Interupt handler for outside a command.
 *
 ****************************************************************************/

void Do_Quit()
{
}

void Do_refresh_number()
{
int i;
int j;
WIDGET *wdgt;
	
for (i = 0; i < activelist.count; i++) {
    wdgt=activelist.widgets[i];
    j=0;
    while(wdgt[j].type != FINISH) {
	if (wdgt[j].callch=='*') {
    	    (void) mvwprintw(wdgt[j].wndw, 1, 1, "%s %s", wdgt[j].label,
		wdgt[j].dstr);	 
	    }
	j++;
	}
    }
}
